"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var chai_1 = require("chai");
var index_1 = require("../src/index");
describe('mapState()', function () {
    it('should return the first mapping found of a state', function () {
        var mapping = index_1.mapState({
            a: 'state a',
            b: 'state b'
        }, 'b');
        chai_1.assert.strictEqual(mapping, 'state b');
    });
    it('should return undefined for unmapped states', function () {
        var mapping = index_1.mapState({
            a: 'state a',
            b: 'state b'
        }, 'c');
        chai_1.assert.isUndefined(mapping);
    });
    it('should prioritize returning equivalent state mapping', function () {
        var mapping = index_1.mapState({
            a: 'state a',
            b: 'state b',
            'b.b1': 'st b.b1'
        }, 'b.b1');
        chai_1.assert.strictEqual(mapping, 'st b.b1');
    });
    it('should return superstate mapping when substate is not found', function () {
        var mapping = index_1.mapState({
            a: 'state a',
            b: 'state b',
            'b.b1': 'st b.b1'
        }, 'b.foo');
        chai_1.assert.strictEqual(mapping, 'state b');
    });
    it('should return superstate mapping when deep substate is not found', function () {
        var mapping = index_1.mapState({
            a: 'state a',
            b: 'state b',
            'b.b1': 'state b.b1'
        }, 'b.b1.foo');
        chai_1.assert.strictEqual(mapping, 'state b.b1');
    });
});
